//=============================================================================
//
//  RecordFinder plugin for MuseScore
//  http://www.musescore.org/en/project/recordfinder
//
//  Copyright (C)2010 Joachim Ganseman, with a HUGE thanks to Nicolas Froment and his SoundCloud plugin
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

//
//    This is ECMAScript code (ECMA-262 aka "Java Script")
//

// ==== INCLUSION OF LAST.FM API ==== //

/*
 *
 * Copyright (c) 2008-2009, Felix Bruns <felixbruns@web.de>
 *
 */

function LastFM(options){
	/* Set default values for required options. */
	var apiKey    = options.apiKey    || '';
	var apiSecret = options.apiSecret || '';
	var apiUrl    = options.apiUrl    || 'http://ws.audioscrobbler.com/2.0/';
	var cache     = options.cache     || undefined;
	
	/* Set API key. */
	this.setApiKey = function(_apiKey){
		apiKey = _apiKey;
	};
	
	/* Set API key. */
	this.setApiSecret = function(_apiSecret){
		apiSecret = _apiSecret;
	};
	
	/* Set API URL. */
	this.setApiUrl = function(_apiUrl){
		apiUrl = _apiUrl;
	};
	
	/* Set cache. */
	this.setCache = function(_cache){
		cache = _cache;
	};
	
	/* Internal call (POST, GET). */
	var internalCall = function(params, callbacks, requestMethod){
		/* Cross-domain POST request (doesn't return any data, always successful). */
		if(requestMethod == 'POST'){
			/* Create iframe element to post data. */
			var html   = document.getElementsByTagName('html')[0];
			var iframe = document.createElement('iframe');
			var doc;
			
			/* Set iframe attributes. */
			iframe.width        = 1;
			iframe.height       = 1;
			iframe.style.border = 'none';
			iframe.onload       = function(){
				/* Remove iframe element. */
				//html.removeChild(iframe);
				
				/* Call user callback. */
				if(typeof(callbacks.success) != 'undefined'){
					callbacks.success();
				}
			};
			
			/* Append iframe. */
			html.appendChild(iframe);
			
			/* Get iframe document. */
			if(typeof(iframe.contentWindow) != 'undefined'){
				doc = iframe.contentWindow.document;
			}
			else if(typeof(iframe.contentDocument.document) != 'undefined'){
				doc = iframe.contentDocument.document.document;
			}
			else{
				doc = iframe.contentDocument.document;
			}
			
			/* Open iframe document and write a form. */
			doc.open();
			doc.clear();
			doc.write('<form method="post" action="' + apiUrl + '" id="form">');
			
			/* Write POST parameters as input fields. */
			for(var param in params){
				doc.write('<input type="text" name="' + param + '" value="' + params[param] + '">');
			}
			
			/* Write automatic form submission code. */
			doc.write('</form>');
			doc.write('<script type="application/x-javascript">');
			doc.write('document.getElementById("form").submit();');
			doc.write('</script>');
			
			/* Close iframe document. */
			doc.close();
		}
		/* Cross-domain GET request (JSONP). */
		else{
			/* Get JSONP callback name. */
			var jsonp = 'jsonp' + new Date().getTime();
			
			/* Calculate cache hash. */
			var hash = auth.getApiSignature(params);
			
			/* Check cache. */
			if(typeof(cache) != 'undefined' && cache.contains(hash) && !cache.isExpired(hash)){
				if(typeof(callbacks.success) != 'undefined'){
					callbacks.success(cache.load(hash));
				}
				
				return;
			}
			
			/* Set callback name and response format. */
			params.callback = jsonp;
			params.format   = 'json';
			
			/* Create JSONP callback function. */
			window[jsonp] = function(data){
				/* Is a cache available?. */
				if(typeof(cache) != 'undefined'){
					var expiration = cache.getExpirationTime(params);
					
					if(expiration > 0){
						cache.store(hash, data, expiration);
					}
				}
				
				/* Call user callback. */
				if(typeof(data.error) != 'undefined'){
					if(typeof(callbacks.error) != 'undefined'){
						callbacks.error(data.error, data.message);
					}
				}
				else if(typeof(callbacks.success) != 'undefined'){
					callbacks.success(data);
				}
				
				/* Garbage collect. */
				window[jsonp] = undefined;
				
				try{
					delete window[jsonp];
				}
				catch(e){
					/* Nothing. */
				}
				
				/* Remove script element. */
				if(head){
					head.removeChild(script);
				}
			};
			
			/* Create script element to load JSON data. */
			var head   = document.getElementsByTagName("head")[0];
			var script = document.createElement("script");
			
			/* Build parameter string. */
			var array = [];
			
			for(var param in params){
				array.push(encodeURIComponent(param) + "=" + encodeURIComponent(params[param]));
			}
			
			params = array.join('&').replace(/%20/g, '+');
			
			/* Set script source. */
			script.src = apiUrl + '?' + params;
			
			/* Append script element. */
			head.appendChild(script);
		}
	};
	
	/* Normal method call. */
	var call = function(method, params, callbacks, requestMethod){
		/* Set default values. */
		params        = params        || {};
		callbacks     = callbacks     || {};
		requestMethod = requestMethod || 'GET';
		
		/* Add parameters. */
		params.method  = method;
		params.api_key = apiKey;
		
		/* Call method. */
		internalCall(params, callbacks, requestMethod);
	};
	
	/* Signed method call. */
	var signedCall = function(method, params, session, callbacks, requestMethod){
		/* Set default values. */
		params        = params        || {};
		callbacks     = callbacks     || {};
		requestMethod = requestMethod || 'GET';
		
		/* Add parameters. */
		params.method  = method;
		params.api_key = apiKey;
		
		/* Add session key. */
		if(session && typeof(session.key) != 'undefined'){
			params.sk = session.key;
		}
		
		/* Get API signature. */
		params.api_sig = auth.getApiSignature(params);
		
		/* Call method. */
		internalCall(params, callbacks, requestMethod);
	};
	
	/* Album methods. */
	this.album = {
		addTags : function(params, session, callbacks){
			/* Build comma separated tags string. */
			if(typeof(params.tags) == 'object'){
				params.tags = params.tags.join(',');
			}
			
			signedCall('album.addTags', params, session, callbacks, 'POST');
		},
		
		getInfo : function(params, callbacks){
			call('album.getInfo', params, callbacks);
		},
		
		getTags : function(params, session, callbacks){
			signedCall('album.getTags', params, session, callbacks);
		},
		
		removeTag : function(params, session, callbacks){
			signedCall('album.removeTag', params, session, callbacks, 'POST');
		},
		
		search : function(params, callbacks){
			call('album.search', params, callbacks);
		}
	};
	
	/* Artist methods. */
	this.artist = {
		addTags : function(params, session, callbacks){
			/* Build comma separated tags string. */
			if(typeof(params.tags) == 'object'){
				params.tags = params.tags.join(',');
			}
			
			signedCall('artist.addTags', params, session, callbacks, 'POST');
		},
		
		getEvents : function(params, callbacks){
			call('artist.getEvents', params, callbacks);
		},
		
		getImages : function(params, callbacks){
			call('artist.getImages', params, callbacks);
		},
		
		getInfo : function(params, callbacks){
			call('artist.getInfo', params, callbacks);
		},
		
		getPastEvents : function(params, callbacks){
			call('artist.getPastEvents', params, callbacks);
		},
		
		getPodcast : function(params, callbacks){
			call('artist.getPodcast', params, callbacks);
		},
		
		getShouts : function(params, callbacks){
			call('artist.getShouts', params, callbacks);
		},
		
		getSimilar : function(params, callbacks){
			call('artist.getSimilar', params, callbacks);
		},
		
		getTags : function(params, session, callbacks){
			signedCall('artist.getTags', params, session, callbacks);
		},
		
		getTopAlbums : function(params, callbacks){
			call('artist.getTopAlbums', params, callbacks);
		},
		
		getTopFans : function(params, callbacks){
			call('artist.getTopFans', params, callbacks);
		},
		
		getTopTags : function(params, callbacks){
			call('artist.getTopTags', params, callbacks);
		},
		
		getTopTracks : function(params, callbacks){
			call('artist.getTopTracks', params, callbacks);
		},
		
		removeTag : function(params, session, callbacks){
			signedCall('artist.removeTag', params, session, callbacks, 'POST');
		},
		
		search : function(params, callbacks){
			call('artist.search', params, callbacks);
		},
		
		share : function(params, session, callbacks){
			/* Build comma separated recipients string. */
			if(typeof(params.recipient) == 'object'){
				params.recipient = params.recipient.join(',');
			}
			
			signedCall('artist.share', params, session, callbacks, 'POST');
		},
		
		shout : function(params, session, callbacks){
			signedCall('artist.shout', params, session, callbacks, 'POST');
		}
	};
	
	/* Auth methods. */
	this.auth = {
		getMobileSession : function(params, callbacks){
			/* Set new params object with authToken. */
			params = {
				username  : params.username,
				authToken : md5(params.username + md5(params.password))
			};
			
			signedCall('auth.getMobileSession', params, null, callbacks);
		},
		
		getSession : function(params, callbacks){
			signedCall('auth.getSession', params, null, callbacks);
		},
		
		getToken : function(callbacks){
			signedCall('auth.getToken', null, null, callbacks);
		},
		
		/* Deprecated. Security hole was fixed. */
		getWebSession : function(callbacks){
			/* Save API URL and set new one (needs to be done due to a cookie!). */
			var previuousApiUrl = apiUrl;
			
			apiUrl = 'http://ext.last.fm/2.0/';
			
			signedCall('auth.getWebSession', null, null, callbacks);
			
			/* Restore API URL. */
			apiUrl = previuousApiUrl;
		}
	};
	
	/* Event methods. */
	this.event = {
		attend : function(params, session, callbacks){
			signedCall('event.attend', params, session, callbacks, 'POST');
		},
		
		getAttendees : function(params, session, callbacks){
			call('event.getAttendees', params, callbacks);
		},
		
		getInfo : function(params, callbacks){
			call('event.getInfo', params, callbacks);
		},
		
		getShouts : function(params, callbacks){
			call('event.getShouts', params, callbacks);
		},
		
		share : function(params, session, callbacks){
			/* Build comma separated recipients string. */
			if(typeof(params.recipient) == 'object'){
				params.recipient = params.recipient.join(',');
			}
			
			signedCall('event.share', params, session, callbacks, 'POST');
		},
		
		shout : function(params, session, callbacks){
			signedCall('event.shout', params, session, callbacks, 'POST');
		}
	};
	
	/* Geo methods. */
	this.geo = {
		getEvents : function(params, callbacks){
			call('geo.getEvents', params, callbacks);
		},
		
		getMetroArtistChart : function(params, callbacks){
			call('geo.getMetroArtistChart', params, callbacks);
		},
		
		getMetroTrackChart : function(params, callbacks){
			call('geo.getMetroTrackChart', params, callbacks);
		},
		
		getMetroUniqueArtistChart : function(params, callbacks){
			call('geo.getMetroUniqueArtistChart', params, callbacks);
		},
		
		getMetroUniqueTrackChart : function(params, callbacks){
			call('geo.getMetroUniqueTrackChart', params, callbacks);
		},
		
		getMetroWeeklyChartlist : function(params, callbacks){
			call('geo.getMetroWeeklyChartlist', params, callbacks);
		},
		
		getTopArtists : function(params, callbacks){
			call('geo.getTopArtists', params, callbacks);
		},
		
		getTopTracks : function(params, callbacks){
			call('geo.getTopTracks', params, callbacks);
		}
	};
	
	/* Group methods. */
	this.group = {
		getMembers : function(params, callbacks){
			call('group.getMembers', params, callbacks);
		},
		
		getWeeklyAlbumChart : function(params, callbacks){
			call('group.getWeeklyAlbumChart', params, callbacks);
		},
		
		getWeeklyArtistChart : function(params, callbacks){
			call('group.group.getWeeklyArtistChart', params, callbacks);
		},
		
		getWeeklyChartList : function(params, callbacks){
			call('group.getWeeklyChartList', params, callbacks);
		},
		
		getWeeklyTrackChart : function(params, callbacks){
			call('group.getWeeklyTrackChart', params, callbacks);
		}
	};
	
	/* Library methods. */
	this.library = {
		addAlbum : function(params, session, callbacks){
			signedCall('library.addAlbum', params, session, callbacks, 'POST');
		},
		
		addArtist : function(params, session, callbacks){
			signedCall('library.addArtist', params, session, callbacks, 'POST');
		},
		
		addTrack : function(params, session, callbacks){
			signedCall('library.addTrack', params, session, callbacks, 'POST');
		},
		
		getAlbums : function(params, callbacks){
			call('library.getAlbums', params, callbacks);
		},
		
		getArtists : function(params, callbacks){
			call('library.getArtists', params, callbacks);
		},
		
		getTracks : function(params, callbacks){
			call('library.getTracks', params, callbacks);
		}
	};
	
	/* Playlist methods. */
	this.playlist = {
		addTrack : function(params, session, callbacks){
			signedCall('playlist.addTrack', params, session, callbacks, 'POST');
		},
		
		create : function(params, session, callbacks){
			signedCall('playlist.create', params, session, callbacks, 'POST');
		},
		
		fetch : function(params, callbacks){
			call('playlist.fetch', params, callbacks);
		}
	};
	
	/* Radio methods. */
	this.radio = {
		getPlaylist : function(params, session, callbacks){
			signedCall('radio.getPlaylist', params, session, callbacks);
		},
		
		tune : function(params, session, callbacks){
			signedCall('radio.tune', params, session, callbacks);
		}
	};
	
	/* Tag methods. */
	this.tag = {
		getSimilar : function(params, callbacks){
			call('tag.getSimilar', params, callbacks);
		},
		
		getTopAlbums : function(params, callbacks){
			call('tag.getTopAlbums', params, callbacks);
		},
		
		getTopArtists : function(params, callbacks){
			call('tag.getTopArtists', params, callbacks);
		},
		
		getTopTags : function(callbacks){
			call('tag.getTopTags', null, callbacks);
		},
		
		getTopTracks : function(params, callbacks){
			call('tag.getTopTracks', params, callbacks);
		},
		
		getWeeklyArtistChart : function(params, callbacks){
			call('tag.getWeeklyArtistChart', params, callbacks);
		},
		
		getWeeklyChartList : function(params, callbacks){
			call('tag.getWeeklyChartList', params, callbacks);
		},
		
		search : function(params, callbacks){
			call('tag.search', params, callbacks);
		}
	};
	
	/* Tasteometer method. */
	this.tasteometer = {
		compare : function(params, callbacks){
			call('tasteometer.compare', params, callbacks);
		}
	};
	
	/* Track methods. */
	this.track = {
		addTags : function(params, session, callbacks){
			signedCall('track.addTags', params, session, callbacks, 'POST');
		},
		
		ban : function(params, session, callbacks){
			signedCall('track.ban', params, session, callbacks, 'POST');
		},
		
		getInfo : function(params, callbacks){
			call('track.getInfo', params, callbacks);
		},
		
		getSimilar : function(params, callbacks){
			call('track.getSimilar', params, callbacks);
		},
		
		getTags : function(params, session, callbacks){
			signedCall('track.getTags', params, session, callbacks);
		},
		
		getTopFans : function(params, callbacks){
			call('track.getTopFans', params, callbacks);
		},
		
		getTopTags : function(params, callbacks){
			call('track.getTopTags', params, callbacks);
		},
		
		love : function(params, session, callbacks){
			signedCall('track.love', params, session, callbacks, 'POST');
		},
		
		removeTag : function(params, session, callbacks){
			signedCall('track.removeTag', params, session, callbacks, 'POST');
		},
		
		search : function(params, callbacks){
			call('track.search', params, callbacks);
		},
		
		share : function(params, session, callbacks){
			/* Build comma separated recipients string. */
			if(typeof(params.recipient) == 'object'){
				params.recipient = params.recipient.join(',');
			}
			
			signedCall('track.share', params, session, callbacks, 'POST');
		}
	};
	
	/* User methods. */
	this.user = {
		getEvents : function(params, callbacks){
			call('user.getEvents', params, callbacks);
		},
		
		getFriends : function(params, callbacks){
			call('user.getFriends', params, callbacks);
		},
		
		getInfo : function(params, callbacks){
			call('user.getInfo', params, callbacks);
		},
		
		getLovedTracks : function(params, callbacks){
			call('user.getLovedTracks', params, callbacks);
		},
		
		getNeighbours : function(params, callbacks){
			call('user.getNeighbours', params, callbacks);
		},
		
		getPastEvents : function(params, callbacks){
			call('user.getPastEvents', params, callbacks);
		},
		
		getPlaylists : function(params, callbacks){
			call('user.getPlaylists', params, callbacks);
		},
		
		getRecentStations : function(params, session, callbacks){
			signedCall('user.getRecentStations', params, session, callbacks);
		},
		
		getRecentTracks : function(params, callbacks){
			call('user.getRecentTracks', params, callbacks);
		},
		
		getRecommendedArtists : function(params, session, callbacks){
			signedCall('user.getRecommendedArtists', params, session, callbacks);
		},
		
		getRecommendedEvents : function(params, session, callbacks){
			signedCall('user.getRecommendedEvents', params, session, callbacks);
		},
		
		getShouts : function(params, callbacks){
			call('user.getShouts', params, callbacks);
		},
		
		getTopAlbums : function(params, callbacks){
			call('user.getTopAlbums', params, callbacks);
		},
		
		getTopArtists : function(params, callbacks){
			call('user.getTopArtists', params, callbacks);
		},
		
		getTopTags : function(params, callbacks){
			call('user.getTopTags', params, callbacks);
		},
		
		getTopTracks : function(params, callbacks){
			call('user.getTopTracks', params, callbacks);
		},
		
		getWeeklyAlbumChart : function(params, callbacks){
			call('user.getWeeklyAlbumChart', params, callbacks);
		},
		
		getWeeklyArtistChart : function(params, callbacks){
			call('user.getWeeklyArtistChart', params, callbacks);
		},
		
		getWeeklyChartList : function(params, callbacks){
			call('user.getWeeklyChartList', params, callbacks);
		},
		
		getWeeklyTrackChart : function(params, callbacks){
			call('user.getWeeklyTrackChart', params, callbacks);
		},
		
		shout : function(params, session, callbacks){
			signedCall('user.shout', params, session, callbacks, 'POST');
		}
	};
	
	/* Venue methods. */
	this.venue = {
		getEvents : function(params, callbacks){
			call('venue.getEvents', params, callbacks);
		},
		
		getPastEvents : function(params, callbacks){
			call('venue.getPastEvents', params, callbacks);
		},
		
		search : function(params, callbacks){
			call('venue.search', params, callbacks);
		}
	};
	
	/* Private auth methods. */
	var auth = {
		getApiSignature : function(params){
			var keys   = [];
			var string = '';
			
			for(var key in params){
				keys.push(key);
			}
			
			keys.sort();
			
			for(var index in keys){
				var key = keys[index];
				
				string += key + params[key];
			}
			
			string += apiSecret;
			
			/* Needs lastfm.api.md5.js. */
			return md5(string);
		}
	};
}


// ==== END INCLUSION OF LAST.FM API ==== //
		
					
// ==== BEGINNING OF PLUGIN ==== //					
					
					
//---------------------------------------------------------
//    init
//    this function will be called on startup of
//    mscore
//---------------------------------------------------------
					
function init()
{
	// print("test script init");
}					
					
					
//---------------------------------------------------------
//    run
//    this function will be called when activating the
//    plugin menu entry
//
//    global Variables:
//    pluginPath - contains the plugin path; file separator
//                 is "/"
//---------------------------------------------------------

var answerBuffer;
var resultId;
// var dirSettings = QDir.homePath() + "/.musescore/plugins/recordfinder";
// var fileSettings = "settings.txt";

function run()
  {
	displaySearch();				
  }
  
					
					
// function to display the search form
function displaySearch(){
					var loader = new QUiLoader(null);
					var file = new QFile(pluginPath + "/recordFinderSearch.ui");
					file.open(QIODevice.OpenMode(QIODevice.ReadOnly, QIODevice.Text));
					form = loader.load(file, null);
					form.formLayoutWidget.lineEditTitle.text = curScore.title;
					form.formLayoutWidget.lineEditSubtitle.text = curScore.subtitle;
					form.formLayoutWidget.lineEditComposer.text = curScore.composer;
					form.formLayoutWidget.lineEditLyricist.text = curScore.poet;
					form.buttonBox.accepted.connect(searchLastFM);
					form.show();  					
}
						

// search LAST.FM API call
function searchLastFM() {

	// initial failed approach
	// LAST.FM JavaScript API bindings are meant for inclusion in websites -> does not work here, depend on a browser window				
					
	/* 
	var cache = new LastFMCache();

	var lastfm = new LastFM({
							apiKey : 'f21088bf9097b49ad4e7f487abab981e',
							apiSecret : '7ccaec2093e33cded282ec7bc81c6fca' //,
							//cache : cache
		});
					
	var searchArtist = "";
					if (form.formLayoutWidget.checkBoxComposer.checked) { searchArtist = searchArtist + form.formLayoutWidget.lineEditComposer.text; }
					if (form.formLayoutWidget.checkBoxLyricist.checked) { searchArtist = searchArtist + " " + form.formLayoutWidget.lineEditLyricist.text; }
	var searchTitle = "";
					if (form.formLayoutWidget.checkBoxTitle.checked) { searchTitle = searchTitle + form.formLayoutWidget.lineEditTitle.text; }
					if (form.formLayoutWidget.checkBoxSubtitle.checked) { searchTitle = searchTitle + " " + form.formLayoutWidget.lineEditSubtitle.text; }
					if (form.formLayoutWidget.checkBoxAdditional.checked) { searchTitle = searchTitle + " " + form.formLayoutWidget.textEditAddition.text; }
					 
	// form.close; QMessageBox.critical(0, "Apicall", "Searchtitle " + searchTitle + "  ;   SearchArtist" + searchArtist);				
	
	// if there is a song title, search for the track name
					lastfm.track.search({
										track: searchTitle,
										artist: searchArtist
										}, 
										{ success: function(data) {displayResults(data);}, 
										error: function(code,message) {displayError(code, message);}
										} );
	*/
					
					
	//new approach using YQL
					
	var URLbase="http://query.yahooapis.com/v1/public/yql?q=";
	var apiKey = "aa1a6a25005a577f19d2c3fcd271e747";
	var apiSecret = "7ccaec2093e33cded282ec7bc81c6fca";
	var hostApi = "query.yahooapis.com"
					
	var searchArtist = "";
					if (form.formLayoutWidget.checkBoxComposer.checked) { searchArtist = searchArtist + form.formLayoutWidget.lineEditComposer.text; }
					if (form.formLayoutWidget.checkBoxLyricist.checked) { searchArtist = searchArtist + " " + form.formLayoutWidget.lineEditLyricist.text; }
	var searchTitle = "";
					if (form.formLayoutWidget.checkBoxTitle.checked) { searchTitle = searchTitle + form.formLayoutWidget.lineEditTitle.text; }
					if (form.formLayoutWidget.checkBoxSubtitle.checked) { searchTitle = searchTitle + " " + form.formLayoutWidget.lineEditSubtitle.text; }
					if (form.formLayoutWidget.checkBoxAdditional.checked) { searchTitle = searchTitle + " " + form.formLayoutWidget.textEditAddition.text; }
	
					
		// necessary: api_key and track fields. artist is optional			
	var YQLquery= 'SELECT * FROM lastfm.track.search WHERE api_key="' + apiKey + '"';
		YQLquery = YQLquery + ' AND track="' + searchTitle + '"'; 
			if (searchArtist) { YQLquery = YQLquery + ' AND artist="' + searchArtist + '"'; }
					

	answerBuffer = new QBuffer();
	answerBuffer.open(QIODevice.OpenMode(QIODevice.ReadWrite));		
	var http = new QHttp();
	http.setHost(hostApi);				
	http.requestFinished.connect(answerBuffer, displayResults);				
					

	var url = URLbase + escape(YQLquery) + '&diagnostics=true&env=' + encodeURIComponent('http://datatables.org/alltables.env');	;

	resultId = http.get(url, answerBuffer);	
						
}
					

function displayResults(id ,error){
	if (id == resultId){
		if(!error){
			form.close(); 
					
			// open new form
			var loader = new QUiLoader(null);
			var file   = new QFile(pluginPath + "/recordFinderResults.ui");
			file.open(QIODevice.OpenMode(QIODevice.ReadOnly, QIODevice.Text));
			form = loader.load(file, null);
			form.buttonBox.accepted.connect(close);		
			form.setWindowFlags(Qt.CustomizeWindowHint);
			form.listView.text = "";		
					//form.progressBar.value = 0;
							
			// read the result list		
			answerBuffer.seek(0);
			var reader = new QXmlStreamReader;
			reader.setDevice(answerBuffer);
			var permalink = "";
					
			// TODO: replace this with JSON
			
			while (!reader.atEnd())
				{
				if (reader.isStartElement()) 
					{
					if (reader.name() == "url") 
						{
							// add to a list of hyperlinks in the UI form
							permalink = reader.readElementText();
							if (permalink.substr(0,18) == "http://www.last.fm") {
								form.listView.text += ("<a href='" + permalink + "'>" + permalink + "</a> <br />"); }
								// print(permalink);
						}
					}
					reader.readNext(); 
				}	
				
			if (form.listView.text == "") { form.listView.text = "No Results Found"; }		
					
			answerBuffer.close();
					
			// show the results form		
			form.show();		
					
		}
		else
		{ displayError(resultId, "FAIL"); }
	}
					
}
					
					
function displayError(code, message)	{
					
	QMessageBox.critical(0, "error! ", code + " " + message)

}
					

function close()
	{
		form.close();
	}
					


//---------------------------------------------------------
//    menu:  defines were the function will be placed
//           in the menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.RecordFinder',
      init: init,
      run:  run
      };

mscorePlugin;

